@@ -102,6 +102,20 @@ class TextClassificationModelLogger(BaseGalileoModelLogger):
         """
         return GalileoModelLoggerAttributes.get_valid()
 
+    def _has_len(self, arr: Any) -> bool:
+        """Checks if an array has length
+
+        Array can be list, numpy array, or tensorflow tensor. Tensorflow tensors don't
+        let you call len(), they throw a TypeError so we catch that here and check
+        shape https://github.com/tensorflow/tensorflow/blob/master/tensorflow/...
+        python/framework/ops.py#L929
+        """
+        try:
+            has_len = len(arr) != 0
+        except TypeError:
+            has_len = bool(arr.shape[0])
+        return has_len
+
     def validate(self) -> None:
         """
         Validates that the current config is correct.
@@ -109,22 +123,24 @@ class TextClassificationModelLogger(BaseGalileoModelLogger):
         :return:
         """
         get_dq_logger().info("Handling logits and probs", split=self.split)
-        if len(self.logits):
+        has_logits = self._has_len(self.logits)
+        has_probs = self._has_len(self.probs)
+        if has_logits:
             self.logits = self._convert_tensor_ndarray(self.logits, "Prob")
             self.probs = self.convert_logits_to_probs(self.logits)
             del self.logits
-        elif len(self.probs):
+        elif has_probs:
             warnings.warn("Usage of probs is deprecated, use logits instead")
             self.probs = self._convert_tensor_ndarray(self.probs, "Prob")
 
-        embs_len = len(self.embs)
-        probs_len = len(self.probs)
-        ids_len = len(self.ids)
-
         get_dq_logger().info("Converting inputs to numpy arrays", split=self.split)
         self.embs = self._convert_tensor_ndarray(self.embs, "Embedding")
         self.ids = self._convert_tensor_ndarray(self.ids)
 
+        embs_len = len(self.embs)
+        probs_len = len(self.probs)
+        ids_len = len(self.ids)
+
         get_dq_logger().info("Validating embedding shape", split=self.split)
         assert self.embs.ndim == 2, "Only one embedding vector is allowed per input."
 
